@@ -13,11 +13,11 @@ SQL::Inserter - Efficient buffered DBI inserter and fast INSERT SQL builder
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.00_3';
+our $VERSION = '0.02_1';
 
 our @EXPORT_OK = qw(simple_insert multi_insert_sql);
 
@@ -28,8 +28,8 @@ our @EXPORT_OK = qw(simple_insert multi_insert_sql);
   my $sql = SQL::Inserter->new(
     dbh    => $dbh,
     table  => 'table',
-    cols   => [qw/column1 column2.../]?,
-    buffer => 100?                       # No. of rows for multi-row insert
+    cols   => [qw/col1 col2.../],
+    buffer => 100?   # Default buffer is 100 rows
   );
 
   # Fastest method: pass single or multiple rows of data as an array
@@ -39,8 +39,8 @@ our @EXPORT_OK = qw(simple_insert multi_insert_sql);
   # (otherwise there is auto-flush on the object's destruction)
   $sql->insert();
 
-  # Alternative, pass a single row as a hash, allows SQL code passed as references
-  # instead of values
+  # Alternative, pass a single row as a hash, allows SQL code passed as
+  # references instead of values (no need to define cols in constructor)
   $sql->insert({
     column1 => $data1,
     column2 => \'NOW()',
@@ -52,7 +52,7 @@ our @EXPORT_OK = qw(simple_insert multi_insert_sql);
   my ($sql, @bind) = simple_insert($table, {col1=>$val...});
 
   # Multi-row variant:
-  my ($sql, @bind) = simple_insert($table, [{col1=>$row1_val...},{col1=>$row2_val...},...]);
+  my ($sql, @bind) = simple_insert($table, [{col1=>$val1...},{col1=>$val2...},...]);
 
   # Or, construct an SQL statement with placeholders for a given number of rows:
   my $sql = multi_insert_sql('table', [qw/col1 col2.../], $num_of_rows);
@@ -70,6 +70,8 @@ hashes (allows SQL code apart from plain values).
 It also provides lightweight functions that return the SQL queries to be used manually,
 similar to C<SQL::Abstract::insert>, but much faster.
 
+C<INSERT IGNORE> and C<ON DUPLICATE KEY UPDATE> variants supported for MySQL/MariaDB.
+
 =head1 EXPORTS
 
 On request: C<simple_insert> C<multi_insert_sql>.
@@ -142,6 +144,13 @@ sub new {
     $self->{buffer} = $args{buffer} || 100;
     $self->{dupes}  = $args{duplicates};
     $self->{null}   = $args{null_undef};
+    $self->{oracle} = $args{oracle};
+    $self->{oracle} = 1
+        if !defined $self->{oracle}
+        && $self->{dbh}->{Driver}->{Name}
+        && $self->{dbh}->{Driver}->{Name} =~ /oracle/i;
+
+    $self->{dupes} = 'oracle' if $self->{oracle};
     if ($self->{dupes}) {
         $self->{ignore} = 1 if $self->{dupes} eq "ignore";
         $self->{update} = 1 if $self->{dupes} eq "update";
@@ -211,35 +220,34 @@ sub insert {
 
     return $self->_hash_insert(@_) if $_[0] and ref($_[0]);
 
+    # Empty the buffer with no argument
+    return $self->{buffer_counter} ? $self->_empty_buffer() : 0 unless @_;
+
+    croak("Calling insert without a hash requires cols defined in constructor")
+        unless $self->{cols};
+
+    croak("Insert arguments must be multiple of cols")
+        if scalar(@_) % scalar @{$self->{cols}};
+
+    croak("Insert was previously called with hash argument (still in buffer)")
+        if $self->{hash_buffer};
+
     my $ret = 0;
-    if (@_) {
+    while (@_) {
+        my $rows = scalar(@_) / scalar @{$self->{cols}};
+        my $left = $self->{buffer} - $self->{buffer_counter}; # Space left in buffer
 
-        croak("Calling insert without a hash requires cols defined in constructor")
-            unless $self->{cols};
-    
-        croak("Insert arguments must be multiple of cols")
-            if scalar(@_) % scalar @{$self->{cols}};
-
-        croak("Insert was previously called with hash argument (still in buffer)")
-            if $self->{hash_buffer};
-
-        while (@_) {
-            my $rows = scalar(@_) / scalar @{$self->{cols}};
-            my $left = $self->{buffer} - $self->{buffer_counter}; # Space left in buffer
-
-            if ($rows > $left) { # Can't fit buffer
-                my $max = $left * scalar @{$self->{cols}};
-                push @{$self->{bind}}, splice(@_,0,$max);
-                $self->{buffer_counter} = $self->{buffer};
-            } else {
-                push @{$self->{bind}}, splice(@_);
-                $self->{buffer_counter} += $rows;
-            }
-            $ret = $self->_write_full_buffer() if $self->{buffer_counter} == $self->{buffer};
+        if ($rows > $left) { # Can't fit buffer
+            my $max = $left * scalar @{$self->{cols}};
+            push @{$self->{bind}}, splice(@_,0,$max);
+            $self->{buffer_counter} = $self->{buffer};
+        } else {
+            push @{$self->{bind}}, splice(@_);
+            $self->{buffer_counter} += $rows;
         }
-    } elsif ($self->{buffer_counter}) { # Empty the buffer
-        $ret = $self->_empty_buffer();
+        $ret = $self->_write_full_buffer() if $self->{buffer_counter} == $self->{buffer};
     }
+
     return $ret;
 }
 
@@ -313,34 +321,39 @@ sub simple_insert {
     my $table  = shift;
     my $fields = shift;
     my $opt    = shift;
+    my $join   = $opt->{oracle} ? "\n" : ",\n";
 
     my ($placeh, @bind, @cols);
     if (ref($fields) eq 'ARRAY') {
         @cols = keys %{$fields->[0]};
+        my $head = $opt->{oracle} ? _oracle_into_head($table, \@cols) : "";
         my @rows;
         foreach my $f (@$fields) {
-            my ($row, @b) = _row_placeholders($f, \@cols, $opt->{null_undef});
+            my ($row, @b) = _row_placeholders($f, \@cols, $opt->{null_undef}, $head);
             push @rows, $row;
             push @bind, @b;
         }
-        $placeh = join(",\n", @rows);
+        $placeh = join($join, @rows);
     } else {
         @cols = keys %$fields;
-        ($placeh, @bind) = _row_placeholders($fields, \@cols, $opt->{null_undef});
+        my $head = $opt->{oracle} ? _oracle_into_head($table, \@cols) : "";
+        ($placeh, @bind) = _row_placeholders($fields, \@cols, $opt->{null_undef}, $head);
     }
 
     return _create_insert_sql(
         $table, \@cols, $placeh, $opt->{duplicates}
-    ), @bind;
+    ), @bind unless $opt->{oracle};
+
+    return _oracle_create_insert_sql($placeh), @bind;
 }
 
 =head2 multi_insert_sql
 
  my $sql = multi_insert_sql(
      $table,
-     \@columns,         # names of table columns
-     $num_of_rows?,     # default = 1
-     $duplicates?       # can be set as ignore/update in case of duplicate key (MySQL)
+     \@columns,      # names of table columns
+     $num_of_rows?,  # default = 1
+     $variant?       # can be set as 'ignore'/'update' on duplicate key (MySQL) or 'oracle'
  );
 
 Builds bulk insert query (single insert is possible too), with ability for
@@ -357,9 +370,10 @@ Optional parameters:
 for a single row. You can define any number of rows to use with multi-row bind
 variable arrays.
 
-=item * C<$duplicate> : For MySQL, passing C<'ignore'> as the 4th argument returns
+=item * C<$variant> : For MySQL, passing C<'ignore'> as the 4th argument returns
 an C<INSERT IGNORE> query. Passing C<'update'> as the argument returns a query
 containing an `ON DUPLICATE KEY UPDATE` clause (see L</"NOTES"> for further details).
+Passing C<'oracle'> will create the C<INSERT ALL> syntax for pre-23c Oracle DB.
 
 =back
 
@@ -369,18 +383,30 @@ sub multi_insert_sql {
     my $table    = shift;
     my $columns  = shift;
     my $num_rows = shift || 1;
-    my $dupe     = shift;
+    my $variant  = shift;
 
     return unless $table && $columns && @$columns;
+    return _oracle_create_insert_sql(
+        _oracle_insert_into($table, $columns) x $num_rows
+    )
+        if $variant && $variant eq 'oracle';
 
     my $placeholders =
         join(",\n", ('(' . join(',', ('?') x @$columns) . ')') x $num_rows);
 
-    return _create_insert_sql($table, $columns, $placeholders, $dupe);
+    return _create_insert_sql($table, $columns, $placeholders, $variant)
 }
 
 ## Private methods
 
+sub _set_head {
+    my $self = shift;
+    $self->{row_head} =
+        $self->{oracle}
+        ? _oracle_into_head($self->{table}, $self->{cols})
+        : "";
+}
+
 sub _hash_insert {
     my $self   = shift;
     my $fields = shift;
@@ -391,7 +417,9 @@ sub _hash_insert {
 
     $self->{buffer_counter}++;
     $self->{cols} = [keys %$fields] if !defined($self->{cols});
-    my ($row, @bind) = _row_placeholders($fields, $self->{cols}, $self->{null});
+    $self->_set_head unless defined $self->{row_head};
+
+    my ($row, @bind) = _row_placeholders($fields, $self->{cols}, $self->{null}, $self->{row_head});
     push @{$self->{hash_buffer}}, $row;
     push @{$self->{bind}}, @bind;
 
@@ -440,13 +468,13 @@ sub _empty_buffer {
 }
 
 sub _write_hash_buffer {
-    my $self = shift;
-
-    my $placeh = join(",\n", @{$self->{hash_buffer}});
-    my $sth    = $self->{dbh}->prepare(
-        _create_insert_sql(
-            $self->{table}, $self->{cols}, $placeh, $self->{dupe}
-        )
+    my $self   = shift;
+    my $join   = $self->{oracle} ? "\n" : ",\n";
+    my $placeh = join($join, @{$self->{hash_buffer}});
+    my $sth = $self->{dbh}->prepare(
+        $self->{oracle}
+        ? _oracle_create_insert_sql($placeh)
+        : _create_insert_sql($self->{table}, $self->{cols}, $placeh, $self->{dupes})
     );
     $self->_execute($sth);
     $self->_cleanup();
@@ -478,17 +506,36 @@ sub DESTROY {
 
 ## Private functions
 
+sub _oracle_into_head {
+    my $table = shift;
+    my $cols  = shift;
+    return "INTO $table(".join(",", @$cols).") VALUES";
+}
+
+sub _oracle_insert_into {
+    my $table = shift;
+    my $cols  = shift;
+    return _oracle_into_head($table, $cols)."(".join(',', ('?') x @$cols).")\n";
+}
+
+sub _oracle_create_insert_sql {
+    my $placeh = shift;
+    chomp($placeh);
+
+    return "INSERT ALL\n$placeh\nSELECT 1 FROM dual;";
+}
+
 sub _create_insert_sql {
     my $table   = shift;
     my $columns = shift;
     my $placeh  = shift;
-    my $dupe    = shift || "";
+    my $variant = shift || "";
 
-    my $ignore = ($dupe eq 'ignore') ? ' IGNORE' : '';
+    my $ignore = ($variant eq 'ignore') ? ' IGNORE' : '';
     my $cols   = join(',', @$columns);
     my $sql    = "INSERT$ignore INTO $table ($cols)\nVALUES $placeh";
 
-    $sql .= _on_duplicate_key_update($columns) if $dupe eq 'update';
+    $sql .= _on_duplicate_key_update($columns) if $variant eq 'update';
 
     return $sql;
 }
@@ -497,9 +544,9 @@ sub _row_placeholders {
     my $fields = shift;
     my $cols   = shift;
     my $null   = shift;
+    my $head   = shift;
+    my $sql    = $head ? "$head(" : "(";
     my @bind   = ();
-    my $sql    = "(";
-
     my $val;
 
     foreach my $key (@$cols) {
